local UnitGUID = UnitGUID
local GetNumGroupMembers = GetNumGroupMembers
local LibInspect = LibStub and LibStub:GetLibrary("LibInspect")
local Ticker = C_Timer.NewTimer
local GetTime = GetTime
local UnitBuff = UnitBuff
local BreakUpLargeNumbers = BreakUpLargeNumbers

--------Leech Scanning ------------------------------------------------------------------------------
local GUID_LeechCache = {}	 -- {[GUID] = Leech} cache for scanned leech values
local GUIDsLastScan = {}  -- {[GUID] = GetTime()} 
local roster = {}		 -- {unitID = leech} contains only current group members, 'false' if leech is unknown
LeechTrinketCounterRoster = roster
local GUIDtoUnitID = {}
local maxAge = 60*5		-- maximum leech scan age in seconds before rescanning
local tooltip -- = CreateFrame('GameTooltip', 'LeechScanTooltip', UIParent, 'GameTooltipTemplate')

local waitingForScan = false
local waitingForScanTimeOut = 5
local waitingForScanTimer
local function CancelWaitingForScan() 
	waitingForScan = false 
	if waitingForScanTimer then 
		waitingForScanTimer:Cancel() 
	end
end
local function WaitForScan()
	waitingForScan = true
	waitingForScanTimer = Ticker(waitingForScanTimeOut, CancelWaitingForScan, 1)
end

local debug = false
local function DebugPrint(...)
	if debug then
		print(...)
	end
end

local function GroupChangedUpdateUnit(u)
	local GUID = UnitGUID(u)
	GUIDtoUnitID[GUID] = u
	local cached = GUID_LeechCache[GUID]
	if cached then
		roster[GUID] = cached
	else
		roster[GUID] = false
	end
end

local f = CreateFrame("frame")
f:RegisterEvent("GROUP_ROSTER_UPDATE")
f:RegisterEvent("PLAYER_ENTERING_WORLD")
f:RegisterEvent("PLAYER_LOGIN")

f:SetScript("OnEvent", function(_, e, ...)
	if e == "GROUP_ROSTER_UPDATE" or e == "PLAYER_ENTERING_WORLD" then
		wipe(roster)
		wipe(GUIDtoUnitID)
		if IsInRaid() then
			for i=1, GetNumGroupMembers() do
				GroupChangedUpdateUnit("raid"..i)
			end
		else
			GroupChangedUpdateUnit("player")
			for i=1, GetNumGroupMembers()-1 do
				GroupChangedUpdateUnit("party"..i)
			end
		end
	elseif e == "PLAYER_LOGIN" then
		tooltip = CreateFrame('GameTooltip', 'LeechScanTooltip', nil, 'GameTooltipTemplate')
		tooltip:SetOwner(UIParent, "ANCHOR_NONE")
	end
end)

local ItemLeechCache = {} -- [hyperlink] = leech
local function ScanItemTooltips(GUID, data, age)
	DebugPrint("received data for scanning")
	if data then
		local leechTotal = 0
		for k, v in ipairs(data.items) do
			local leech = ItemLeechCache[v]
			if not leech then
				leech = 0
				tooltip:ClearLines()
				tooltip:SetHyperlink(v) -- example hyperlink with leech "|cffa335ee|Hitem:124244:0:0:0:0:0:0:0:100:270:0:5:2:41:566|h[Chestguard of Gnawing Desire]|h|r"
				for i=1, tooltip:NumLines() do
					leech = leech + (tonumber(_G["LeechScanTooltipTextLeft"..i]:GetText():match("+(%d+) Leech")) or 0) 
				end
				ItemLeechCache[v] = leech
			end
			leechTotal = leechTotal + leech
		end
		DebugPrint(GUIDtoUnitID[GUID], leechTotal)
		roster[GUID] = leechTotal
		GUID_LeechCache[GUID] = leechTotal
		GUIDsLastScan[GUID] = GetTime()
	end
	CancelWaitingForScan()
end

if not LibInspect or not LibInspect:AddHook("LeechTrinketCounter", "items", function(GUID, data, age) ScanItemTooltips(GUID, data, age) end) then
	error("LeechTrinketCounter: Error loading LibInspect")
	return
else
	LibInspect:SetMaxAge(maxAge)
end

local function QueueUnitScan(GUID)
	local UnitID = GUIDtoUnitID[GUID]
	if not UnitIsVisible(UnitID) then 
		DebugPrint("can't scan", (UnitName(GUIDtoUnitID[GUID])))
		return false 
	end
	caninspect, unitfound, refreshing = LibInspect:RequestData("items", UnitID, false)
	local waiting = caninspect and unitfound and refreshing
	WaitForScan(waiting)
	DebugPrint((waiting and "scanning" or "can't scan"), (UnitName(GUIDtoUnitID[GUID])))
	return waiting
end

local scanTick = C_Timer.NewTicker(2, function()
	if InCombatLockdown() then return end
	if waitingForScan then --DebugPrint("waiting") 
		DebugPrint("waiting for scan")
		return end
	for k, v in pairs(roster) do 
		if not v then 
			if QueueUnitScan(k) then
				return
			end
		end
	end
	local time = GetTime()
	for k, v in pairs(roster) do 
		if GUIDsLastScan[k] and GUIDsLastScan[k] + maxAge < time then 
			if QueueUnitScan(k) then
				return
			end
		end
	end
	DebugPrint("no scanning neccesary or availible")
end)

------------CombatLog Analysing ----------------------------------------------------------------------
local timestamp
local inCombat = false
local combatStartTime
local boss1Name
local trinketSources = {}
local currentCombatStats = {}
local pasive, active = {}, {}

local c = CreateFrame("frame")
c:SetScript("OnEvent", function(_, e, time, subevent, _, _, _, _, _, destGUID, targetName, _, _, spellID, _, _, amount, overhealing)
	if spellID == 143924 then
		local staticLeech = roster[destGUID] or 0
		local totalLeech = staticLeech
		wipe(trinketSources)
		if overhealing == amount then return end
		for i=1, 40 do
			local name, _, _, _, _, _, _, unitID, _, _, spellID, _, _, _, leech = UnitAura(targetName, i)
			if not name then
				break
			end
			if spellID == 184671 then
				trinketSources[UnitGUID(unitID or "") or "UNKNOWN"] = leech
				totalLeech = totalLeech + leech
			end
		end
		
		if overhealing > 0 then --assign maximum possible part of effective healing to static leech, the rest is divided normaly
			local pasiveLeechFraction = staticLeech/totalLeech
			local effectiveHealing = amount-overhealing
			local pasivePart = pasiveLeechFraction <= overhealing/amount and pasiveLeechFraction*effectiveHealing or effectiveHealing
			pasive[destGUID] = (pasive[destGUID] or 0) + pasivePart
			amount = amount - pasivePart
			totalLeech = totalLeech - staticLeech
			if amount > 0 then
				for k, v in pairs(trinketSources) do
					active[k] = active[k] or {[true] = 0, [false] = 0}
					active[k][k == destGUID] = active[k][k == destGUID] + (v/totalLeech)*amount
				end
			end
		else
			pasive[destGUID] = (pasive[destGUID] or 0) + (staticLeech/totalLeech)*amount
			for k, v in pairs(trinketSources) do
				active[k] = active[k] or {[true] = 0, [false] = 0}
				active[k][k == destGUID] = active[k][k == destGUID] + (v/totalLeech)*amount
			end
		end
	end
end)

local function CombatSectionStart()
	if not inCombat then
		inCombat = true
		DebugPrint("combat start")
		wipe(currentCombatStats)
		currentCombatStats.pasive = {}
		currentCombatStats.active = {}
		pasive = currentCombatStats.pasive
		active = currentCombatStats.active
		combatStartTime = GetTime()
		c:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	end
end

local function CombatSectionEnd()
	if inCombat then
		for k, v in pairs(active) do
			print(select(6,GetPlayerInfoByGUID(k)), "- on others:", BreakUpLargeNumbers(floor(v[false])), " on self:", BreakUpLargeNumbers(floor(v[true])))
		end
		inCombat = false
		DebugPrint("combat end")
		c:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	end
end

local d = CreateFrame("frame")
d:RegisterEvent("PLAYER_REGEN_ENABLED")
d:RegisterEvent("PLAYER_REGEN_DISABLED")
d:RegisterEvent("ENCOUNTER_START")
d:RegisterEvent("ENCOUNTER_END")
d:SetScript("OnEvent", function(_, e, ...)
	--[[if InCombatLockdown() then
		CombatSectionStart()
	else
		CombatSectionEnd()
	end]]
	if e == "PLAYER_REGEN_DISABLED" or e == "ENCOUNTER_START" then
		CombatSectionStart()
	elseif e == "PLAYER_REGEN_ENABLED" or e == "ENCOUNTER_END" then
		CombatSectionEnd()
	end
end)

SLASH_LEECHTRINKETCOUNTER1 = "/ltc"
SlashCmdList["LEECHTRINKETCOUNTER"] = function(msg)
	for k, v in pairs(active) do
		local str = {select(6,GetPlayerInfoByGUID(k)), "- on others:", BreakUpLargeNumbers(floor(v[false])), " on self:", BreakUpLargeNumbers(floor(v[true]))}
		local chat
		if IsInGroup(LE_PARTY_CATEGORY_INSTANCE) then
			chat = "INSTANCE_CHAT"
		elseif IsInRaid() then
			chat = "RAID"
		else
			chat = "PARTY"
		end
		SendChatMessage(table.concat(str, " "), chat)
	end
end